#Script for performing K-means clustering and Rank-size analyses

#before starting set your working directory

#install required packages
install.packages("rgdal")
install.packages("cluster")
install.packages("factoextra")
install.packages("vegclust")

#Load data
library(rgdal)
mydata<-readOGR("shp/khabur_triangle.shp",layer="khabur_triangle") # load shape file
mydata<-mydata@data #convert shape file into a data frame

# Calculate the within groups sum of squared error (SSE) for the number of cluster solutions selected by the user
set.seed(1234)
wss <- (nrow(mydata[,4:5])-1)*sum(apply(mydata[,4:5],2,var))
for (i in 2:15) wss[i] <- sum(kmeans(mydata[,4:5], centers=i, nstart=100)$withinss) #loop for calculating the SSE for each cluster solution ranging from 2 to 15

# Calculate the within groups SSE for 1000 randomized data sets (based on the original input data)
k.rand <- function(x){
  randx<-sample((x)[,1])
  randy<-sample((x)[,2])
  rand<-data.frame(randx,randy)
  rand<-as.matrix(rand)
  rand.wss <- (nrow(rand)-1)*sum(apply(rand,2,var))
  for (i in 2:15) rand.wss[i] <- sum(kmeans(rand, centers=i, nstart=100)$withinss) 
  return(rand.wss)}
rand.mat <- matrix(0,15,1000)
k.1 <- function(x){
  for (i in 1:1000) {
    r.1 <- k.rand(mydata[,4:5])
    rand.mat[,i] <- r.1}
  return(rand.mat)}
rand.mat <- k.1(mydata.m)

# Plot within groups SSE against all tested cluster solutions for the observed and randomised data 
layout(matrix(c(1,1,2,3), 2, 2, byrow = TRUE))
par(mar=c(4, 4, 1,0.5))
par(ask=TRUE) 
xrange <- range(1:15)
yrange <- range(log(rand.mat),log(wss))
#Plot log scale
plot(xrange,yrange, type='n', xlab='Number of Clusters (k)', ylab='Log of within cluster SSE', main='K-means clustering solutions')
for (i in 1:1000) lines(log(rand.mat[,i]),type='l',col='grey') #plot envelope of randomised data
lines(log(wss), type="b", pch=19,col='black')
legend('top',c('observed data', '1000 random runs'), col=c('black', 'grey'), lty=1, bty="n")
#Plot normal scale
par(ask=TRUE)
xrange <- range(1:8)
yrange <- range(1:80)
plot(xrange,yrange, type='n', xlab='Number of Clusters (k)', ylab='Within cluster SSE')
for (i in 1:1000) lines(rand.mat[,i],type='l',col='grey') #plot envelope of randomised data
lines(wss, type="b", pch=19,col='black')

# Compute Average silhouette width for K-means clustering
#1st method
library(cluster)
sil <- rep(0, 15) #define the number of cluster solutions
for(i in 2:15){
  km.res <- kmeans(mydata[,4:5], centers = i, nstart = 25)
  ss <- silhouette(km.res$cluster, dist(mydata[,4:5]))
  sil[i] <- mean(ss[,3])
}
# Plot the  average silhouette width
plot(1:15, sil, type = "b", pch = 19, xlab = "Number of Clusters (k)", ylab="Avg. silhouette width")
abline(v = which.max(sil), lty = 2)

# alternatively you can use the package "factoextra"
library(factoextra)
fviz_nbclust(mydata[,4:5], kmeans, method = "silhouette",linecolor="black")

#Fuzzy K-means
library(vegclust)
Cl <- vegclust(mydata[,4:5], 4, method = "FCM", nstart = 100) #define the number of clusters 
cl.mem <- Cl$memb
maxmem <- apply(cl.mem, 1, max) #Define cluster's membership of each point (based on the highest fuzzy score for each cluster)
cl.out <- defuzzify(Cl)$cluster #Crisp clustering
cl.out <- ifelse(maxmem < 0.5, 0, cl.out) # Define noise: all points with less than 50% of probability of being part of a cluster
clname <- data.frame(as.factor(cl.out)) #convert points'id from character to factor, and then from factor into a data.frame
cl_mydata <- cbind(mydata, clname)   #attach points'id to the original data.frame 
colnames(cl_mydata)[6]<-"Cluster" #rename the columns of data.frame 
#Plot Fuzzy K-means 
plot(cl_mydata$X, cl_mydata$Y, col=cl_mydata$Cluster, axes=TRUE, pch=19, xlab='eastings', ylab='northings')


#PART 2
#RANK-SIZE ANALYSIS WITHIN EACH CLUSTER
############################################
#Extract sites size for each cluster. (Remember to repeat the steps below for each different cluster M1,M2,...)
Cl1<-subset(cl_mydata, cl_mydata[,3] & cl_mydata$Cluster=="M1")
sizeCl1<-Cl1[,3]  
 
#source the codes:
source("rank_size/a12coeff.R") 
source("rank_size/bootStrap_acoeff12.R") 

#calculate the observed A-coefficient (with plot):
a12coeff(sizeCl1,plotting=TRUE) 

#assess the 95 confidence envelope using 1000 simulation bootstrap:
#the automatically plotted histogram shows the distrubution of the simulated A coefficients, along with the observed one (the red line)
envelopeCl1<-bootStrap_acoeff12(sizeCl1,nsim=1000) 

#obtain the 95% confidence envelopes
envelopeCl1$envA 
envelopeCl1$envA1 
envelopeCl1$envA2 


